@@ -2,7 +2,8 @@
  * Seahorse
  * 
  * Copyright (C) 2008 Stefan Walter
- * 
+ * Copyright (C) 2011 Collabora Ltd.
+ *
  * This program is free software; you can redistribute it and/or modify 
  * it under the terms of the GNU Lesser General Public License as
  * published by the Free Software Foundation; either version 2.1 of
@@ -26,10 +27,11 @@
 #include "seahorse-gkr.h"
 #include "seahorse-gkr-keyring.h"
 #include "seahorse-gkr-dialogs.h"
+#include "seahorse-gkr-operation.h"
 #include "seahorse-gkr-source.h"
 
-#include "common/seahorse-registry.h"
-
+#include "seahorse-progress.h"
+#include "seahorse-registry.h"
 #include "seahorse-source.h"
 #include "seahorse-util.h"
 
@@ -62,9 +64,11 @@ static void on_view_selection_changed (SeahorseView *view, gpointer user_data);
  */
 
 static void
-on_refresh_done (SeahorseOperation *op, gpointer user_data)
+on_refresh_all_keyrings_complete (GObject *source,
+                                  GAsyncResult *result,
+                                  gpointer user_data)
 {
-	SeahorseCommands *self = user_data;
+	SeahorseCommands *self = SEAHORSE_COMMANDS (user_data);
 	SeahorseView *view;
 
 	g_return_if_fail (SEAHORSE_IS_COMMANDS (self));
@@ -77,21 +81,11 @@ on_refresh_done (SeahorseOperation *op, gpointer user_data)
 }
 
 static void
-refresh_all_keyrings (SeahorseCommands *self)
+refresh_all_keyrings (SeahorseCommands *commands)
 {
-	SeahorseOperation *op;
-
-	op = seahorse_source_load (SEAHORSE_SOURCE (seahorse_gkr_source_default ()));
-	
-	/* 
-	 * HACK: Major hack alert. This whole area needs some serious refactoring,
-	 * so for now we're just going to let any viewers listen in on this
-	 * operation like so:
-	 */
-	g_signal_emit_by_name (seahorse_context_instance (), "refreshing", op);
-
-	seahorse_operation_watch (op, on_refresh_done, g_object_ref (self), NULL, NULL);
-	g_object_unref (op);
+	seahorse_source_load_async (SEAHORSE_SOURCE (seahorse_gkr_source_default ()),
+	                            NULL, on_refresh_all_keyrings_complete,
+	                            g_object_ref (commands));
 }
 
 /**
@@ -125,26 +119,23 @@ static const GtkActionEntry ENTRIES_NEW[] = {
 static void
 on_keyring_unlock_done (GnomeKeyringResult result, gpointer user_data)
 {
-	SeahorseCommands *self = SEAHORSE_COMMANDS (user_data);
-	SeahorseView *view;
+	SeahorseCommands *commands = SEAHORSE_COMMANDS (user_data);
 
 	if (result != GNOME_KEYRING_RESULT_OK &&
 	    result != GNOME_KEYRING_RESULT_DENIED &&
 	    result != GNOME_KEYRING_RESULT_CANCELLED) {
-		view = seahorse_commands_get_view (self);
-		seahorse_util_show_error (GTK_WIDGET (seahorse_view_get_window (view)),
+		seahorse_util_show_error (GTK_WIDGET (seahorse_commands_get_window (commands)),
 		                          _("Couldn't unlock keyring"),
 		                          gnome_keyring_result_to_message (result));
 	}
-	
-	refresh_all_keyrings (self);
+
+	refresh_all_keyrings (commands);
 }
 
 static void 
 on_keyring_unlock (GtkAction *action, SeahorseGkrKeyringCommands *self)
 {
 	SeahorseView *view;
-	SeahorseGkrKeyring *keyring;
 	GList *keys, *l;
 
 	g_return_if_fail (SEAHORSE_IS_GKR_KEYRING_COMMANDS (self));
@@ -154,7 +145,6 @@ on_keyring_unlock (GtkAction *action, SeahorseGkrKeyringCommands *self)
 	keys = seahorse_view_get_selected_matching (view, &keyring_predicate);
 
 	for (l = keys; l; l = g_list_next (l)) {
-		keyring = SEAHORSE_GKR_KEYRING (l->data);
 		g_return_if_fail (SEAHORSE_IS_GKR_KEYRING (l->data));
 		gnome_keyring_unlock (seahorse_gkr_keyring_get_name (l->data), NULL, 
 		                      on_keyring_unlock_done, g_object_ref (self), g_object_unref);
@@ -166,26 +156,23 @@ on_keyring_unlock (GtkAction *action, SeahorseGkrKeyringCommands *self)
 static void
 on_keyring_lock_done (GnomeKeyringResult result, gpointer user_data)
 {
-	SeahorseCommands *self = SEAHORSE_COMMANDS (user_data);
-	SeahorseView *view;
+	SeahorseCommands *commands = SEAHORSE_COMMANDS (user_data);
 
 	if (result != GNOME_KEYRING_RESULT_OK &&
 	    result != GNOME_KEYRING_RESULT_DENIED &&
 	    result != GNOME_KEYRING_RESULT_CANCELLED) {
-		view = seahorse_commands_get_view (self);
-		seahorse_util_show_error (GTK_WIDGET (seahorse_view_get_window (view)),
+		seahorse_util_show_error (GTK_WIDGET (seahorse_commands_get_window (commands)),
 		                          _("Couldn't lock keyring"),
 		                          gnome_keyring_result_to_message (result));
 	}
 
-	refresh_all_keyrings (self);
+	refresh_all_keyrings (commands);
 }
 
 static void 
 on_keyring_lock (GtkAction *action, SeahorseGkrKeyringCommands *self)
 {
 	SeahorseView *view;
-	SeahorseGkrKeyring *keyring;
 	GList *keyrings, *l;
 
 	g_return_if_fail (SEAHORSE_IS_GKR_KEYRING_COMMANDS (self));
@@ -195,7 +182,6 @@ on_keyring_lock (GtkAction *action, SeahorseGkrKeyringCommands *self)
 	keyrings = seahorse_view_get_selected_matching (view, &keyring_predicate);
 
 	for (l = keyrings; l; l = g_list_next (l)) {
-		keyring = SEAHORSE_GKR_KEYRING (l->data);
 		g_return_if_fail (SEAHORSE_IS_GKR_KEYRING (l->data));
 		gnome_keyring_lock (seahorse_gkr_keyring_get_name (l->data), 
 		                    on_keyring_lock_done, g_object_ref (self), g_object_unref);
@@ -207,19 +193,17 @@ on_keyring_lock (GtkAction *action, SeahorseGkrKeyringCommands *self)
 static void
 on_set_default_keyring_done (GnomeKeyringResult result, gpointer user_data)
 {
-	SeahorseCommands *self = SEAHORSE_COMMANDS (user_data);
-	SeahorseView *view;
+	SeahorseCommands *commands = SEAHORSE_COMMANDS (user_data);
 
 	if (result != GNOME_KEYRING_RESULT_OK &&
 	    result != GNOME_KEYRING_RESULT_DENIED &&
 	    result != GNOME_KEYRING_RESULT_CANCELLED) {
-		view = seahorse_commands_get_view (self);
-		seahorse_util_show_error (GTK_WIDGET (seahorse_view_get_window (view)),
+		seahorse_util_show_error (GTK_WIDGET (seahorse_commands_get_window (commands)),
 		                          _("Couldn't set default keyring"),
 		                          gnome_keyring_result_to_message (result));
 	}
-	
-	refresh_all_keyrings (self);
+
+	refresh_all_keyrings (commands);
 }
 
 static void
@@ -246,26 +230,23 @@ on_keyring_default (GtkAction *action, SeahorseGkrKeyringCommands *self)
 static void
 on_change_password_done (GnomeKeyringResult result, gpointer user_data)
 {
-	SeahorseCommands *self = SEAHORSE_COMMANDS (user_data);
-	SeahorseView *view;
+	SeahorseCommands *commands = SEAHORSE_COMMANDS (user_data);
 
 	if (result != GNOME_KEYRING_RESULT_OK &&
 	    result != GNOME_KEYRING_RESULT_DENIED &&
 	    result != GNOME_KEYRING_RESULT_CANCELLED) {
-		view = seahorse_commands_get_view (self);
-		seahorse_util_show_error (GTK_WIDGET (seahorse_view_get_window (view)),
+		seahorse_util_show_error (GTK_WIDGET (seahorse_commands_get_window (commands)),
 		                          _("Couldn't change keyring password"),
 		                          gnome_keyring_result_to_message (result));
 	}
-	
-	refresh_all_keyrings (self);
+
+	refresh_all_keyrings (commands);
 }
 
 static void
 on_keyring_password (GtkAction *action, SeahorseGkrKeyringCommands *self)
 {
 	SeahorseView *view;
-	SeahorseGkrKeyring *keyring;
 	GList *keys, *l;
 
 	g_return_if_fail (SEAHORSE_IS_GKR_KEYRING_COMMANDS (self));
@@ -275,7 +256,6 @@ on_keyring_password (GtkAction *action, SeahorseGkrKeyringCommands *self)
 	keys = seahorse_view_get_selected_matching (view, &keyring_predicate);
 
 	for (l = keys; l; l = g_list_next (l)) {
-		keyring = SEAHORSE_GKR_KEYRING (l->data);
 		g_return_if_fail (SEAHORSE_IS_GKR_KEYRING (l->data));
 		gnome_keyring_change_password (seahorse_gkr_keyring_get_name (l->data), NULL, NULL,
 		                               on_change_password_done, g_object_ref (self), g_object_unref);
@@ -340,7 +320,7 @@ seahorse_gkr_keyring_commands_show_properties (SeahorseCommands* base, SeahorseO
 	g_return_if_fail (SEAHORSE_IS_OBJECT (object));
 	g_return_if_fail (seahorse_object_get_tag (object) == SEAHORSE_GKR_TYPE);
 
-	window = seahorse_view_get_window (seahorse_commands_get_view (base));
+	window = seahorse_commands_get_window (base);
 	if (G_OBJECT_TYPE (object) == SEAHORSE_TYPE_GKR_KEYRING) 
 		seahorse_gkr_keyring_properties_show (SEAHORSE_GKR_KEYRING (object), window);
 	
@@ -348,24 +328,50 @@ seahorse_gkr_keyring_commands_show_properties (SeahorseCommands* base, SeahorseO
 		g_return_if_reached ();
 }
 
-static SeahorseOperation* 
-seahorse_gkr_keyring_commands_delete_objects (SeahorseCommands* base, GList* objects) 
+static void
+on_delete_objects_complete (GObject *source, GAsyncResult *result, gpointer user_data)
 {
-	SeahorseOperation *oper = NULL;
+	SeahorseCommands *commands = SEAHORSE_COMMANDS (user_data);
+	GError *error = NULL;
+	GtkWidget *parent;
+
+	if (!seahorse_gkr_delete_finish (result, &error)) {
+		parent = GTK_WIDGET (seahorse_commands_get_window (commands));
+		seahorse_util_show_error (parent, _("Couldn't delete keyring"), error->message);
+		g_error_free (error);
+	}
+
+	g_object_unref (commands);
+}
+
+static gboolean
+seahorse_gkr_keyring_commands_delete_objects (SeahorseCommands* commands,
+                                              GList* objects)
+{
+	GCancellable *cancellable;
 	gchar *prompt;
-	
+	gboolean ret;
+
 	if (!objects)
-		return NULL;
+		return TRUE;
 
 	prompt = g_strdup_printf (_ ("Are you sure you want to delete the password keyring '%s'?"), 
 	                          seahorse_object_get_label (objects->data));
 
-	if (seahorse_util_prompt_delete (prompt, GTK_WIDGET (seahorse_commands_get_window (base))))
-		oper = seahorse_object_delete (objects->data);
-		
+	ret = seahorse_util_prompt_delete (prompt, GTK_WIDGET (seahorse_commands_get_window (commands)));
+
+	if (ret) {
+		cancellable = g_cancellable_new ();
+		seahorse_gkr_delete_async (objects, cancellable,
+		                           on_delete_objects_complete,
+		                           g_object_ref (commands));
+		seahorse_progress_show (cancellable, ngettext ("Deleting keyring", "Deleting keyrings",
+		                                               g_list_length (objects)), TRUE);
+		g_object_unref (cancellable);
+	}
+
 	g_free (prompt);
-	
-	return oper;
+	return ret;
 }
 
 static GObject* 
@@ -433,7 +439,7 @@ seahorse_gkr_keyring_commands_class_init (SeahorseGkrKeyringCommandsClass *klass
 
 	cmd_class->show_properties = seahorse_gkr_keyring_commands_show_properties;
 	cmd_class->delete_objects = seahorse_gkr_keyring_commands_delete_objects;
-	
+
 	g_type_class_add_private (gobject_class, sizeof (SeahorseGkrKeyringCommandsPrivate));
 
 	/* Setup the predicate for these commands */
