@@ -38,7 +38,7 @@
 #define DEBUG_FLAG SEAHORSE_DEBUG_DNSSD
 #include "seahorse-debug.h"
 
-#define HKP_SERVICE_TYPE "_pgpkey-hkp._tcp."
+#define HKP_SERVICE_TYPE "_pgpkey-hkp._tcp"
 
 /* 
  * DEBUG: Define if we want to show discoveries from this
@@ -414,6 +414,7 @@ seahorse_service_discovery_list (SeahorseServiceDiscovery *ssd)
 	while (g_hash_table_iter_next (&iter, &key, NULL))
 		g_ptr_array_add (result, g_strdup (key));
 
+	g_ptr_array_add (result, NULL);
 	return (gchar **)g_ptr_array_free (result, FALSE);
 }
 
