@@ -3,6 +3,7 @@
  *
  * Copyright (C) 2003 Jacob Perkins
  * Copyright (C) 2004-2005 Stefan Walter
+ * Copyright (C) 2011 Collabora Ltd.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,9 +23,9 @@
 
 #include "config.h"
 
-#include "seahorse-gconf.h"
 #include "seahorse-object.h"
 #include "seahorse-util.h"
+#include "seahorse-widget.h"
 
 #include "pgp/seahorse-pgp-module.h"
 
@@ -68,11 +69,11 @@ void
 seahorse_util_show_error (GtkWidget *parent, const gchar *heading, const gchar *message)
 {
 	GtkWidget *dialog;
-    
+
 	g_return_if_fail (message || heading);
 	if (!message)
 		message = "";
-    
+
 	if (parent) {
 		if (!GTK_IS_WIDGET (parent)) {
 			g_warn_if_reached ();
@@ -105,32 +106,50 @@ seahorse_util_show_error (GtkWidget *parent, const gchar *heading, const gchar *
 
 /**
  * seahorse_util_handle_error:
- * @err: The #GError to print.
+ * @error: The #GError to print, and clear
  * @desc: The heading of the box
  * @...: Parameters to insert into the format string desc.
  *
  * Displays an error box. The message is the error message.
+ * Won't display cancel errors.
  */
 void
-seahorse_util_handle_error (GError* err, const char* desc, ...)
+seahorse_util_handle_error (GError **error,
+                            gpointer parent,
+                            const char* description,
+                            ...)
 {
-    gchar *t = NULL;
-    
-    va_list ap;
-    
-    if(!err)
-        return;
+	gchar *text = NULL;
+	GtkWidget *widget = NULL;
+	va_list ap;
 
-    va_start(ap, desc);
-  
-    if (desc)
-        t = g_strdup_vprintf (desc, ap);
-    
-    va_end(ap);        
-    
-    seahorse_util_show_error (NULL, t, err->message ? err->message : "");
-    g_free(t);
-}    
+	if (!error || !(*error) ||
+	    g_error_matches (*error, G_IO_ERROR, G_IO_ERROR_CANCELLED)) {
+		g_clear_error (error);
+		return;
+	}
+
+	va_start (ap, description);
+	if (description)
+		text = g_strdup_vprintf (description, ap);
+	va_end (ap);
+
+	if (parent == NULL)
+		widget = NULL;
+	else if (GTK_IS_WIDGET (parent))
+		widget = GTK_WIDGET (parent);
+	else if (GTK_IS_WINDOW (parent))
+		widget = GTK_WIDGET (parent);
+	else if (SEAHORSE_IS_WIDGET (parent))
+		widget = seahorse_widget_get_toplevel (parent);
+	else
+		g_warning ("unsupported 'parent' argument passed to seahorse_util_handle_error() ");
+
+	seahorse_util_show_error (widget, text,
+	                          (*error)->message ? (*error)->message : "");
+	g_free (text);
+	g_clear_error (error);
+}
 
 /**
  * seahorse_util_prompt_delete:
@@ -241,48 +260,6 @@ seahorse_util_get_display_date_string (const time_t time)
 	return created_string;
 }
 
-/**
- * seahorse_util_get_text_view_text:
- * @view: The text view #GtkTextView to extract text from
- *
- * Returns the whole text buffer
- *
- * Returns: The text buffer extracted. The returned string should be freed with 
- * #g_free when no longer needed.
- */
-gchar*
-seahorse_util_get_text_view_text (GtkTextView *view)
-{
-	GtkTextBuffer *buffer;
-	GtkTextIter start;                                                      
-    GtkTextIter end;
-	gchar *text;
-	
-    g_return_val_if_fail (view != NULL, "");	
-	
-	buffer = gtk_text_view_get_buffer (view);                                                
-    gtk_text_buffer_get_bounds (buffer, &start, &end);                      
-    text = gtk_text_buffer_get_text (buffer, &start, &end, FALSE);   
-    return text;
-}
-
-/**
- * seahorse_util_set_text_view_string:
- * @view: The view to set the text for (#GtkTextView)
- * @string: The string to set (#GString)
- *
- *
- */
-void
-seahorse_util_set_text_view_string (GtkTextView *view, GString *string)
-{
-    GtkTextBuffer *buffer;
-	g_return_if_fail (view != NULL && string != NULL);
-	
-	buffer = gtk_text_view_get_buffer (view);
-	gtk_text_buffer_set_text (buffer, string->str, string->len);
-}
-
 /**
  * seahorse_util_read_to_memory:
  * @input: Data to read. The #GInputStream is read till the end.
@@ -310,8 +287,11 @@ seahorse_util_read_to_memory (GInputStream *input, guint *len)
 	string = g_string_new ("");
 	buffer = g_new (gchar, size);
     
-	while (g_input_stream_read_all (input, buffer, size, &nread, NULL, NULL) && nread == size)
+	while (g_input_stream_read_all (input, buffer, size, &nread, NULL, NULL)) {
 		string = g_string_append_len (string, buffer, nread);
+		if (nread != size)
+			break;
+	}
 
 	if (len)
 		*len = string->len;
@@ -377,58 +357,6 @@ seahorse_util_read_data_block (GString *buf, GInputStream *input,
     return copied;
 }
 
-/**
- * seahorse_util_memory_input_string:
- * @string: The string to create the stream from
- * @length: The length of this string
- *
- * Returns: The new input stream of type #GMemoryInputStream
- */
-GMemoryInputStream*
-seahorse_util_memory_input_string (const gchar *string, gsize length)
-{
-	g_return_val_if_fail (string, NULL);
-	return G_MEMORY_INPUT_STREAM (g_memory_input_stream_new_from_data (g_strndup (string, length), length, g_free));
-}
-
-/**
- * seahorse_util_memory_output_length:
- * @output: a stream
- *
- * A replacement for #g_memory_output_stream_get_data_size (since 2.18)
- *
- * Returns: The length of the stream
- */
-gsize
-seahorse_util_memory_output_length (GMemoryOutputStream *output)
-{
-	GSeekable *seekable;
-	goffset offset, end;
-	
-	/* 
-	 * This is a replacement for g_memory_output_stream_get_data_size()
-	 * which is not available in current version of glib.
-	 */
-	
-	g_return_val_if_fail (G_IS_MEMORY_OUTPUT_STREAM (output), 0);
-	g_return_val_if_fail (G_IS_SEEKABLE (output), 0);
-	
-	seekable = G_SEEKABLE (output);
-	offset = g_seekable_tell (seekable);
-	
-	if (!g_seekable_seek (seekable, 0, G_SEEK_END, NULL, NULL))
-		g_return_val_if_reached (0);
-	
-	end = g_seekable_tell (seekable);
-	
-	if (offset != end) {
-		if (!g_seekable_seek (seekable, offset, G_SEEK_SET, NULL, NULL))
-			g_return_val_if_reached (0);
-	}
-	
-	return (gsize)end;
-}
-
 /** 
  * seahorse_util_print_fd:
  * @fd: The file descriptor to write to
@@ -552,28 +480,6 @@ seahorse_util_uri_get_last (const gchar* uri)
     return t;
 }    
 
-/**
- * seahorse_util_uri_split_last:
- * @uri: The uri to split
- * 
- * Splits the uri in two at it's last component. The result
- * is still part of the same string, so don't free it. This 
- * modifies the @uri argument.
- * 
- * Returns: The last component
- **/
-const gchar* 
-seahorse_util_uri_split_last (gchar* uri)
-{
-    gchar* t;
-    
-    t = (gchar*)seahorse_util_uri_get_last (uri);
-    if(t != uri)
-        *(t - 1) = 0;
-        
-    return t;
-}
-
 /**
  * seahorse_util_uri_exists:
  * @uri: The uri to check
@@ -656,135 +562,6 @@ seahorse_util_uri_unique (const gchar* uri)
     return uri_try ? uri_try : g_strdup (uri);       
 }
 
-/**
- * seahorse_util_uri_replace_ext:
- * @uri: The uri with old extension
- * @ext: The new extension
- *
- * Replaces the extension on @uri
- *
- * Returns: Newly allocated URI string with new extension. The returned string
- * should be freed with #g_free when no longer needed.
- **/
-gchar* 
-seahorse_util_uri_replace_ext (const gchar *uri, const gchar *ext)
-{
-    gchar* ret;
-    gchar* dot;
-    gchar* slash;
-    guint len;
-    
-    len = strlen (uri);
-    ret = g_new0 (gchar, len + strlen(ext) + 16);
-    strcpy (ret, uri);
- 
-    /* Always take off a slash at end */
-    g_return_val_if_fail (len > 1, ret);
-    if (ret[len - 1] == '/')
-        ret[len - 1] = 0;
-        
-    dot = strrchr (ret, '.');
-    if (dot != NULL) {
-        slash = strrchr (ret, '/');
-        if (slash == NULL || dot > slash)
-            *dot = 0;
-    }
- 
-    /* Only begin extension with . if provided extension doesn't start with
-       one already. */
-    if(ext[0] != '.')
-        strcat (ret, ".");
-
-    /* Finally append the caller's provided extension. */
-    strcat (ret, ext);
-    return ret;
-}
-
-/**
- * seahorse_util_uris_package:
- * @package: Package uri
- * @uris: null-terminated array of uris to package 
- * 
- * Package uris into an archive. The uris must be local.
- * 
- * Returns: TRUE on success or FALSE on failure
- */
-gboolean
-seahorse_util_uris_package (const gchar* package, const char** uris)
-{
-    GError* err = NULL;
-    gchar *out = NULL;
-    gint status;
-    gboolean r;
-    GString *str;
-    gchar *cmd;
-    gchar *t;
-    gchar *x;
-    GFile *file, *fpackage;
-    
-    	fpackage = g_file_new_for_uri (package);
-    	t = g_file_get_path (fpackage);
-    	x = g_shell_quote (t);
-    	g_free (t);
-    
-    	/* create execution */
-    	str = g_string_new ("");
-    	g_string_printf (str, "file-roller --add-to=%s", x);
-    	g_free (x);
-    
-    	while(*uris) {
-    		/* We should never be passed any remote uris at this point */
-    		x = g_uri_parse_scheme (*uris);
-    		if (x) 
-    			file = g_file_new_for_uri (*uris);
-    		else
-    			file = g_file_new_for_path (*uris);
-    		g_free (x);
-    		
-    		t = g_file_get_path (file);
-    		g_object_unref (file);
-    		g_return_val_if_fail (t != NULL, FALSE);
-
-    		x = g_shell_quote (t);
-    		g_free (t);
-
-    		g_string_append_printf (str, " %s", x);
-    		g_free (x);
-        
-    		uris++;
-    	}
-        
-    	/* Execute the command */
-    	cmd = g_string_free (str, FALSE);
-    	r = g_spawn_command_line_sync (cmd, &out, NULL, &status, &err);
-    	g_free (cmd); 
-    
-    	/* TODO: This won't work for remote packages if we support them in the future */
-    	t = g_file_get_path (fpackage);
-	g_chmod (t, S_IRUSR | S_IWUSR);
-	g_free (t);
-    	g_object_unref (fpackage);
-
-    if (out) {
-        g_print ("%s", out);
-        g_free (out);
-    }
-    
-    if (!r) {
-        seahorse_util_handle_error (err, _("Couldn't run file-roller"));
-        g_clear_error (&err);
-        return FALSE;
-    }
-    
-    if(!(WIFEXITED(status) && WEXITSTATUS(status) == 0)) {
-        seahorse_util_show_error(NULL, _("Couldn't package files"), 
-                                 _("The file-roller process did not complete successfully"));
-        return FALSE;
-    }
-    
-    return TRUE;
-}
-
 /**
  * seahorse_util_detect_mime_type:
  * @mime: The mime string
@@ -811,8 +588,8 @@ seahorse_util_detect_mime_type (const gchar *mime)
 	         g_ascii_strcasecmp (mime, "application/x-pem-key") == 0)
 		return SEAHORSE_SSH;
 #endif 
-    
-	g_warning ("unsupported type of key data: %s", mime);
+
+	g_message ("unsupported type of key data: %s", mime);
 	return 0;
 }
 
@@ -1119,158 +896,6 @@ seahorse_util_chooser_open_prompt (GtkDialog *dialog)
     return uri;
 }
 
-/**
- * seahorse_util_check_suffix:
- * @path: Path of file to check
- * @suffix: Suffix type to check for.
- *
- * Checks that @path has a suffix specified by @suffix.
- *
- * Returns: TRUE if the file has a correct suffix, FALSE otherwise
- **/
-gboolean
-seahorse_util_check_suffix (const gchar *path, SeahorseSuffix suffix)
-{
-	if (suffix == SEAHORSE_SIG_SUFFIX)
-        return g_str_has_suffix (path, SEAHORSE_EXT_SIG) || 
-               g_str_has_suffix (path, SEAHORSE_EXT_ASC);
-	else
-        return g_str_has_suffix (path, SEAHORSE_EXT_PGP) ||
-               g_str_has_suffix (path, SEAHORSE_EXT_GPG) ||
-               g_str_has_suffix (path, SEAHORSE_EXT_ASC);
-}
-
-/**
- * seahorse_util_add_suffix:
- * @path: Path of an existing file
- * @suffix: Suffix type
- * @prompt: Overwrite prompt text
- *
- * Constructs a new path for a file based on @path plus a suffix determined by
- * @suffix. If ASCII Armor is enabled, the suffix will be '.asc'. Otherwise the 
- * suffix will be '.pgp' if @suffix is %SEAHORSE_CRYPT_SUFFIX or '.sig' if 
- * @suffix is %SEAHORSE_SIG_SUFFIX.
- *
- * Returns: A new path with the suffix appended to @path. NULL if prompt cancelled
- **/
-gchar*
-seahorse_util_add_suffix (const gchar *path, SeahorseSuffix suffix, const gchar *prompt)
-{
-    GtkDialog *dialog;
-    const gchar *ext;
-    gchar *uri;
-    gchar *t;
-    
-    if (suffix == SEAHORSE_CRYPT_SUFFIX)
-        ext = SEAHORSE_EXT_PGP;
-    else
-        ext = SEAHORSE_EXT_SIG;
-    
-    uri = g_strdup_printf ("%s%s", path, ext);
-    
-    if (prompt && uri && seahorse_util_uri_exists (uri)) {
-            
-        t = g_strdup_printf (prompt, seahorse_util_uri_get_last (uri));
-        dialog = seahorse_util_chooser_save_new (t, NULL);
-        g_free (t);
-
-        seahorse_util_chooser_show_key_files (dialog);
-        gtk_file_chooser_select_uri (GTK_FILE_CHOOSER (dialog), uri);
-
-        g_free (uri);                
-        uri = NULL;
-            
-        uri = seahorse_util_chooser_save_prompt (dialog);
-        gtk_widget_destroy (GTK_WIDGET (dialog));
-    }
-
-    return uri;         
-}
-
-/**
- * seahorse_util_remove_suffix:
- * @path: Path with a suffix
- * @prompt:Overwrite prompt text
- *
- * Removes a suffix from @path. Does not check if @path actually has a suffix.
- *
- * Returns: @path without a suffix. NULL if prompt cancelled
- **/
-gchar*
-seahorse_util_remove_suffix (const gchar *path, const gchar *prompt)
-{
-    GtkDialog *dialog;
-    gchar *uri;
-    gchar *t;
-    
-    g_return_val_if_fail (path != NULL, NULL);
-    uri =  g_strndup (path, strlen (path) - 4);
-   
-    if (prompt && uri && seahorse_util_uri_exists (uri)) {
-            
-        t = g_strdup_printf (prompt, seahorse_util_uri_get_last (uri));
-        dialog = seahorse_util_chooser_save_new (t, NULL);
-        g_free (t);
-
-        seahorse_util_chooser_show_key_files (dialog);
-		gtk_file_chooser_select_uri (GTK_FILE_CHOOSER (dialog), uri);
-		
-        g_free (uri);                
-        uri = NULL;
-            
-        uri = seahorse_util_chooser_save_prompt (dialog);
-        gtk_widget_destroy (GTK_WIDGET (dialog));
-    }   
-    
-    return uri;
-}
-
-/**
- * seahorse_util_strvec_dup:
- * @vec: the string table to copy
- *
- * Copy a string table
- *
- * Returns: the new char **
- */
-gchar**
-seahorse_util_strvec_dup (const gchar** vec)
-{
-    gint len = 0;
-    gchar** ret;
-    const gchar** v;
-    
-    if (vec) {
-        for(v = vec; *v; v++)
-            len++;
-    }
-   
-    ret = (gchar**)g_new0(gchar*, len + 1);
-
-    while((--len) >= 0)
-        ret[len] = g_strdup(vec[len]);
-    
-    return ret;
-}
-
-/**
- * seahorse_util_strvec_length:
- * @vec: The string table
- *
- * Calculates the length of the string table
- *
- * Returns: The length of the string table
- */
-guint 
-seahorse_util_strvec_length (const gchar **vec)
-{
-    guint len = 0;
-    while (*(vec++))
-        len++;
-    return len;
-}
-
-
 /**
  * sort_objects_by_source:
  * @k1: the first seahorse object
@@ -1373,42 +998,6 @@ seahorse_util_string_equals (const gchar *s1, const gchar *s2)
     return g_str_equal (s1, s2);
 }
 
-/**
- * seahorse_util_string_up_first:
- * @orig: The utf8 string to work with
- *
- * Upper case the first char in the UTF8 string
- *
- * Returns: a new string, with the first char upper cased. The returned string
- * should be freed with #g_free when no longer needed.
- */
-gchar*
-seahorse_util_string_up_first (const gchar *orig)
-{
-    gchar *t, *t2, *ret;
-    
-    if (g_utf8_validate (orig, -1, NULL)) {
-        
-        t = g_utf8_find_next_char (orig, NULL); 
-        if (t != NULL) {
-            t2 = g_utf8_strup (orig, t - orig);
-            ret = g_strdup_printf ("%s%s", t2, t);
-            g_free (t2);
-            
-        /* Can't find first UTF8 char */
-        } else {
-            ret = g_strdup (orig);
-        }
-    
-    /* Just use ASCII functions when not UTF8 */        
-    } else {
-        ret = g_strdup (orig);
-        ret[0] = g_ascii_toupper (ret[0]);
-    }
-    
-    return ret;
-}
-
 /**
  * seahorse_util_string_lower:
  * @s: ASCII string to change
@@ -1422,63 +1011,6 @@ seahorse_util_string_lower (gchar *s)
         *s = g_ascii_tolower (*s);
 }
 
-/**
- * seahorse_util_string_slist_free:
- * @slist: the #GSList to free
- *
- * Free a GSList along with string values
- *
- * Returns: NULL
- */
-GSList*
-seahorse_util_string_slist_free (GSList* list)
-{
-    GSList *l;
-    for (l = list; l; l = l->next)
-        g_free (l->data);
-    g_slist_free (list);
-    return NULL;
-}
-
-/**
- * seahorse_util_string_slist_copy:
- * @slist: The list to copy
- *
- * Copy a #GSList along with string values
- *
- * Returns: the new list
- */
-GSList*
-seahorse_util_string_slist_copy (GSList *list)
-{
-    GSList *l = NULL;
-    for ( ; list; list = g_slist_next(list))
-        l = g_slist_append (l, g_strdup(list->data));
-    return l;
-}
-
-/**
- * seahorse_util_string_slist_equal:
- * @sl1: the first string list
- * @sl2: the second string list
- *
- * Compare two string GSLists.
- *
- * Returns: TRUE if all the string are equal
- */
-gboolean    
-seahorse_util_string_slist_equal (GSList *l1, GSList *l2)
-{
-    while (l1 && l2) {
-        if (!g_str_equal ((const gchar*)l1->data, (const gchar*)l2->data))
-            break;
-        l1 = g_slist_next (l1);
-        l2 = g_slist_next (l2);
-    }
-    
-    return !l1 && !l2;   
-}
-
 /**
  * seahorse_util_string_is_whitespace:
  * @text: The UTF8 string to test
@@ -1586,7 +1118,7 @@ seahorse_util_determine_popup_menu_position (GtkMenu *menu, int *x, int *y,
 
         widget = GTK_WIDGET (gdata);
 
-        gtk_widget_size_request (GTK_WIDGET (menu), &requisition);
+        gtk_widget_get_preferred_size (GTK_WIDGET (menu), &requisition, NULL);
 
         gdk_window_get_origin (gtk_widget_get_window (widget), &menu_xpos, &menu_ypos);
 
@@ -1657,7 +1189,7 @@ free_avahi ()
 }
 
 const AvahiPoll*
-seahorse_util_dns_sd_get_poll ()
+seahorse_util_dns_sd_get_poll (void)
 {
     if (!avahi_poll) {
         
