@@ -2,6 +2,7 @@
  * Seahorse
  *
  * Copyright (C) 2006 Stefan Walter
+ * Copyright (C) 2011 Collabora Ltd.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -28,12 +29,12 @@
  
 #include "egg-datetime.h"
 
-#include "seahorse-widget.h"
-#include "seahorse-util.h"
-#include "seahorse-progress.h"
 #include "seahorse-gtkstock.h"
+#include "seahorse-registry.h"
 #include "seahorse-passphrase.h"
-#include "seahorse-gtkstock.h"
+#include "seahorse-progress.h"
+#include "seahorse-util.h"
+#include "seahorse-widget.h"
 
 #include "seahorse-pgp.h"
 #include "seahorse-gpgme.h"
@@ -42,14 +43,25 @@
 #include "seahorse-gpgme-key-op.h"
 #include "seahorse-gpgme-source.h"
 
-#include "common/seahorse-registry.h"
-
 /**
  * SECTION:seahorse-gpgme-generate
  * @short_description: This file contains creation dialogs for pgp key creation.
  *
  **/
 
+void           on_gpgme_generate_response                    (GtkDialog *dialog,
+                                                              guint response,
+                                                              gpointer user_data);
+
+void           on_gpgme_generate_entry_changed               (GtkEditable *editable,
+                                                              gpointer user_data);
+
+void           on_gpgme_generate_expires_toggled             (GtkToggleButton *button,
+                                                              gpointer user_data);
+
+void           on_gpgme_generate_algorithm_changed           (GtkComboBox *combo,
+                                                              gpointer user_data);
+
 /* --------------------------------------------------------------------------
  * ACTIONS
  */
@@ -126,17 +138,6 @@ static AlgorithmDesc available_algorithms[] = {
  *
  * If the key generation caused errors, these will be displayed in this function
  */
-static void
-completion_handler (SeahorseOperation *op, gpointer data)
-{
-    GError *error = NULL;
-    if (!seahorse_operation_is_successful (op)) {
-        seahorse_operation_copy_error (op, &error);
-        seahorse_util_handle_error (error, _("Couldn't generate PGP key"));
-        g_clear_error (&error);
-    }
-}
-
 
 /**
  * get_expiry_date:
@@ -166,6 +167,16 @@ get_expiry_date (SeahorseWidget *swidget)
     return widget;
 }
 
+static void
+on_generate_key_complete (GObject *source,
+                          GAsyncResult *result,
+                          gpointer user_data)
+{
+	GError *error = NULL;
+
+	if (!seahorse_gpgme_key_op_generate_finish (SEAHORSE_GPGME_SOURCE (source), result, &error))
+		seahorse_util_handle_error (&error, NULL, _("Couldn't generate PGP key"));
+}
 
 /**
  * gpgme_generate_key:
@@ -182,34 +193,34 @@ get_expiry_date (SeahorseWidget *swidget)
  * @expire sets the expiry date
  *
  */
-void seahorse_gpgme_generate_key (SeahorseGpgmeSource *sksrc, const gchar *name, const gchar *email,
-                            const gchar *comment, guint type, guint bits, time_t expires)
+void
+seahorse_gpgme_generate_key (SeahorseGpgmeSource *source,
+                             const gchar *name,
+                             const gchar *email,
+                             const gchar *comment,
+                             guint type,
+                             guint bits,
+                             time_t expires)
 {
-    SeahorseOperation *op;
-    const gchar *pass;
-    gpgme_error_t gerr;
-    GtkDialog *dialog;
+	GCancellable *cancellable;
+	const gchar *pass;
+	GtkDialog *dialog;
 
+	dialog = seahorse_passphrase_prompt_show (_("Passphrase for New PGP Key"),
+	                                          _("Enter the passphrase for your new key twice."),
+	                                          NULL, NULL, TRUE);
+	if (gtk_dialog_run (dialog) == GTK_RESPONSE_ACCEPT) {
+		pass = seahorse_passphrase_prompt_get (dialog);
+		cancellable = g_cancellable_new ();
+		seahorse_gpgme_key_op_generate_async (source, name, email, comment,
+		                                      pass, type, bits, expires,
+		                                      cancellable, on_generate_key_complete,
+		                                      NULL);
+		seahorse_progress_show (cancellable, _("Generating key"), FALSE);
+		g_object_unref (cancellable);
+	}
 
-
-    dialog = seahorse_passphrase_prompt_show (_("Passphrase for New PGP Key"),
-                                              _("Enter the passphrase for your new key twice."),
-                                              NULL, NULL, TRUE);
-    if (gtk_dialog_run (dialog) == GTK_RESPONSE_ACCEPT)
-    {
-        pass = seahorse_passphrase_prompt_get (dialog);
-        op = seahorse_gpgme_key_op_generate (sksrc, name, email, comment,
-                                             pass, type, bits, expires, &gerr);
-
-        if (!GPG_IS_OK (gerr)) {
-            seahorse_gpgme_handle_error (gerr, _("Couldn't generate key"));
-        } else {
-            seahorse_progress_show (op, _("Generating key"), TRUE);
-            seahorse_operation_watch (op, (SeahorseDoneFunc)completion_handler, NULL, NULL, NULL);
-            g_object_unref (op);
-        }
-    }
-    gtk_widget_destroy (GTK_WIDGET (dialog));
+	gtk_widget_destroy (GTK_WIDGET (dialog));
 }
 
 
@@ -224,8 +235,11 @@ void seahorse_gpgme_generate_key (SeahorseGpgmeSource *sksrc, const gchar *name,
  *
  */
 G_MODULE_EXPORT void
-on_gpgme_generate_response (GtkDialog *dialog, guint response, SeahorseWidget *swidget)
+on_gpgme_generate_response (GtkDialog *dialog,
+                            guint response,
+                            gpointer user_data)
 {
+    SeahorseWidget *swidget = SEAHORSE_WIDGET (user_data);
     SeahorseGpgmeSource *sksrc;
     GtkWidget *widget;
     gchar *name;
@@ -316,8 +330,10 @@ on_gpgme_generate_response (GtkDialog *dialog, guint response, SeahorseWidget *s
  *
  */
 G_MODULE_EXPORT void
-on_gpgme_generate_entry_changed (GtkEditable *editable, SeahorseWidget *swidget)
+on_gpgme_generate_entry_changed (GtkEditable *editable,
+                                 gpointer user_data)
 {
+    SeahorseWidget *swidget = SEAHORSE_WIDGET (user_data);
     GtkWidget *widget;
     gchar *name;
 
@@ -341,8 +357,10 @@ on_gpgme_generate_entry_changed (GtkEditable *editable, SeahorseWidget *swidget)
  * Handles the expires toggle button feedback
  */
 G_MODULE_EXPORT void
-on_gpgme_generate_expires_toggled (GtkToggleButton *button, SeahorseWidget *swidget)
+on_gpgme_generate_expires_toggled (GtkToggleButton *button,
+                                   gpointer user_data)
 {
+    SeahorseWidget *swidget = SEAHORSE_WIDGET (user_data);
     GtkWidget *widget;
     
     widget = get_expiry_date (swidget);
@@ -360,8 +378,10 @@ on_gpgme_generate_expires_toggled (GtkToggleButton *button, SeahorseWidget *swid
  *
  */
 G_MODULE_EXPORT void
-on_gpgme_generate_algorithm_changed (GtkComboBox *combo, SeahorseWidget *swidget)
+on_gpgme_generate_algorithm_changed (GtkComboBox *combo,
+                                     gpointer user_data)
 {
+    SeahorseWidget *swidget = SEAHORSE_WIDGET (user_data);
     GtkWidget *widget;
     gint sel;
     
